
import { createContext, useContext, useState, useEffect, ReactNode } from 'react';
import { supabase } from '@/integrations/supabase/client';
import { User, Session } from '@supabase/supabase-js';
import { toast } from 'sonner';

interface AuthContextType {
  user: User | null;
  session: Session | null;
  isAuthenticated: boolean;
  isLoading: boolean;
  emailConfirmed: boolean;
  login: (email: string, password: string) => Promise<{ error: any }>;
  signUp: (email: string, password: string, metadata?: { name?: string }) => Promise<{ error: any, user: User | null, needsEmailConfirmation: boolean }>;
  logout: () => Promise<void>;
  resetPassword: (email: string) => Promise<{ error: any }>;
  updatePassword: (password: string) => Promise<{ error: any }>;
  startVerification: (method: 'email' | 'phone', value: string) => void;
  verificationMethod: 'email' | 'phone' | null;
  verificationValue: string;
  resendVerificationEmail: (email: string) => Promise<{ error: any, data: any }>;
}

const AuthContext = createContext<AuthContextType | undefined>(undefined);

export const useAuth = () => {
  const context = useContext(AuthContext);
  if (context === undefined) {
    throw new Error('useAuth must be used within an AuthProvider');
  }
  return context;
};

interface AuthProviderProps {
  children: ReactNode;
}

const AuthProvider = ({ children }: AuthProviderProps) => {
  const [user, setUser] = useState<User | null>(null);
  const [session, setSession] = useState<Session | null>(null);
  const [isLoading, setIsLoading] = useState(true);
  const [verificationMethod, setVerificationMethod] = useState<'email' | 'phone' | null>(null);
  const [verificationValue, setVerificationValue] = useState('');
  const [emailConfirmed, setEmailConfirmed] = useState(true);

  useEffect(() => {
    // Set up auth state listener
    const { data: { subscription } } = supabase.auth.onAuthStateChange(
      (event, currentSession) => {
        console.log("Auth state change event:", event);
        
        setSession(currentSession);
        setUser(currentSession?.user ?? null);
        setIsLoading(false);
        
        // Handle session refresh errors
        if (event === 'SIGNED_OUT') {
          console.log("User signed out");
        }
      }
    );

    // Get initial session
    const initializeAuth = async () => {
      setIsLoading(true);
      try {
        const { data: { session: initialSession } } = await supabase.auth.getSession();
        console.log("Initial session:", initialSession);
        setSession(initialSession);
        setUser(initialSession?.user ?? null);
      } catch (error) {
        console.error("Error getting initial session:", error);
        toast.error("Authentication error", {
          description: "Failed to retrieve your session. Please try logging in again.",
        });
      } finally {
        setIsLoading(false);
      }
    };

    initializeAuth();

    // Cleanup subscription
    return () => {
      subscription.unsubscribe();
    };
  }, []);

  const login = async (email: string, password: string) => {
    try {
      const { data, error } = await supabase.auth.signInWithPassword({
        email,
        password
      });
      
      console.log("Login response:", data, error);
      
      if (error) {
        if (error.message.includes('Email not confirmed')) {
          setEmailConfirmed(false);
          toast.error("Email not confirmed", {
            description: "Please check your inbox and confirm your email before signing in.",
          });
        } else {
          toast.error("Login failed", {
            description: error.message || "Please check your credentials and try again.",
          });
        }
      }
      
      return { error };
    } catch (error) {
      console.error("Login error:", error);
      return { error };
    }
  };

  const signUp = async (email: string, password: string, metadata?: { name?: string }) => {
    try {
      const { data, error } = await supabase.auth.signUp({
        email,
        password,
        options: {
          data: metadata || {},
          emailRedirectTo: `${window.location.origin}/sign-in`
        }
      });
      
      console.log("Sign up response:", data, error);
      
      const needsEmailConfirmation = !data?.user?.email_confirmed_at;
      
      if (needsEmailConfirmation && !error) {
        setEmailConfirmed(false);
        toast.success("Account created", {
          description: "Please check your email to confirm your account.",
        });
      }
      
      return { error, user: data?.user || null, needsEmailConfirmation };
    } catch (error) {
      console.error("Sign up error:", error);
      return { error, user: null, needsEmailConfirmation: false };
    }
  };

  const resendVerificationEmail = async (email: string) => {
    try {
      const { data, error } = await supabase.auth.resend({
        type: 'signup',
        email,
        options: {
          emailRedirectTo: `${window.location.origin}/sign-in`,
        },
      });
      
      console.log("Resend verification response:", data, error);
      
      if (!error) {
        toast.success("Verification email sent", {
          description: "Please check your inbox and spam folder for the confirmation email.",
        });
      } else {
        toast.error("Failed to resend verification email", {
          description: error.message,
        });
      }
      
      return { data, error };
    } catch (error) {
      console.error("Resend verification error:", error);
      return { data: null, error };
    }
  };

  const logout = async () => {
    try {
      await supabase.auth.signOut();
    } catch (error) {
      console.error("Logout error:", error);
      toast.error("Logout error", {
        description: "There was a problem signing you out. Please try again.",
      });
    }
  };

  const resetPassword = async (email: string) => {
    try {
      const { error } = await supabase.auth.resetPasswordForEmail(email, {
        redirectTo: `${window.location.origin}/update-password`,
      });
      return { error };
    } catch (error) {
      console.error("Reset password error:", error);
      return { error };
    }
  };

  const updatePassword = async (password: string) => {
    try {
      const { error } = await supabase.auth.updateUser({
        password,
      });
      return { error };
    } catch (error) {
      console.error("Update password error:", error);
      return { error };
    }
  };

  const startVerification = (method: 'email' | 'phone', value: string) => {
    setVerificationMethod(method);
    setVerificationValue(value);
  };

  return (
    <AuthContext.Provider
      value={{
        user,
        session,
        isAuthenticated: !!user,
        isLoading,
        emailConfirmed,
        login,
        signUp,
        logout,
        resetPassword,
        updatePassword,
        startVerification,
        verificationMethod,
        verificationValue,
        resendVerificationEmail
      }}
    >
      {children}
    </AuthContext.Provider>
  );
};

export default AuthProvider;
